@@ -293,7 +293,7 @@ class Interface(object):
                          'config', 'customdata', 'customparams', 'notices',
                          'getIsyInfo', 'getAll', 'setLogLevel',
                          'customtypeddata', 'customtypedparams', 'getNsInfo',
-                         'discover', 'nsdata']
+                         'discover', 'nsdata', 'setController']
 
             parsed_msg = json.loads(msg.payload.decode('utf-8'))
             #LOGGER.debug('MQTT Received Message: {}: {}'.format(msg.topic, parsed_msg))
@@ -338,6 +338,21 @@ class Interface(object):
 
                     else:
                         LOGGER.error('set input was not a list')
+                elif key == 'custom':
+                    # custom response, parse and put standard keys in queue
+                    for custom_key in parsed_msg[key]:
+                        if custom_key in inputCmds:
+                            LOGGER.debug('QUEUING incoming message {}'.format(custom_key))
+                            self.inQueue.put(parsed_msg[key])
+                        else:
+                            LOGGER.info('custom data response {}'.format(parsed_msg[key]))
+                            '''
+                            This should be the response to saving user defined
+                            custom data.  Since the node server just wants
+                            this saved, it doesn't make sense to publish this
+                            as an event.  But we could if there's a valid 
+                            reason to do so.
+                            '''
                 elif key == 'installprofile':
                     LOGGER.info('Profile installation finished')
                 elif key == 'error':
@@ -766,6 +781,8 @@ class Interface(object):
 
             except (KeyError, ValueError) as err:
                 LOGGER.error('Failed to set {}: {}'.format(key, err), exc_info=True)
+        elif key == 'setController':
+            LOGGER.debug('connection status node/driver update')
 
     def _handleResult(self, result):
         try:
@@ -815,7 +832,7 @@ class Interface(object):
         """ Tells you if this nodeserver and Polyglot are connected via MQTT """
         return self.connected
 
-    def addNode(self, node):
+    def addNode(self, node, conn_status=None):
         """
         Add a node to the NodeServer
 
@@ -837,6 +854,9 @@ class Interface(object):
         self._nodes[node.address] = node
         self.nodes_internal[node.address] = node
 
+        if conn_status is not None:
+            self.setController(node.address, conn_status)
+
         """
         This is too early to call the node's start function. At this point
         we may not have receiced anything configuration from Polyglot and
@@ -849,6 +869,22 @@ class Interface(object):
         """ Returns a copy of the last config received. """
         return self.config
 
+    def db_getNodeDrivers(self, addr):
+        """ Returns a list of nodes that were saved in the database """
+        nl = []
+        try:
+            if 'nodes' in self.config:
+                nl = self.config['nodes'] # an array of nodes
+                for n in nl:
+                    if n['address'] == addr:
+                        return n['drivers']  # this is an array
+            else:
+                LOGGER.error(f'node {addr} not found in config')
+        except:
+            LOGGER.warning(f'nodeserver config has not yet been recieved.')
+
+        return nl
+
     def getNodes(self):
         """
         Returns your list of nodes.  This is a list of nodes with your
@@ -1026,7 +1062,7 @@ class Interface(object):
             LOGGER.info(
                 'check_profile: Updated needed since it has never been recorded.')
             update_profile = True
-        elif isinstance(cdata, dict) and self.serverdata['profile_version'] == cdata:
+        elif isinstance(cdata, str) and self.serverdata['profile_version'] == cdata:
             LOGGER.info('check_profile: No updated needed: "{}" == "{}"'.format(
                 self.serverdata['profile_version'], cdata))
             update_profile = False
@@ -1042,7 +1078,7 @@ class Interface(object):
 
             st = self.updateProfile()
 
-            self._ifaceData.profile_version = serverdata['profile_version']
+            self._ifaceData.profile_version = self.serverdata['profile_version']
 
         return update_profile
 
@@ -1109,6 +1145,13 @@ class Interface(object):
         LOGGER.debug('Sending message {}'.format(message))
         self.send(message, 'system')
 
+    def setController(self, node_addr, driver):
+        LOGGER.info('Using node "{}", driver "{}" for connection status.'.format(node_addr, driver))
+        message = {
+                'setController': { 'node': node_addr, 'driver': driver }
+                }
+        self.send(message, 'system')
+
     def nodes(self):
         for n in self.nodes_internal:
             yield self.nodes_internal[n]
